/* 
 * 
 * Copyright (C) 2009-2010 Colomban Wendling <ban@herbesfolles.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#include "ctpl.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include "stack.h"


static void
dump_g_memory_output_stream (GMemoryOutputStream *stream,
                             FILE  *fp)
{
  gchar  *buf;
  gsize   buf_size;
  
  buf = g_memory_output_stream_get_data (stream);
#if GLIB_CHECK_VERSION(2,18,0)
  /* FIXME: this function is new in GIO 2.18, how to read GMemoryOutputStream
   * without it!? */
  buf_size = g_memory_output_stream_get_data_size (stream);
#else
  /* hope the allocated size is correct, or the memory is 0-filled (though we
   * know it is not the case...) */
  buf_size = g_memory_output_stream_get_size (stream);
#endif
  if (fwrite (buf, 1, buf_size, fp) < buf_size) {
    fprintf (stderr, "Failed to dump some data: %s\n", g_strerror (errno));
  } else {
    fputc ('\n', fp);
  }
}

int
main (int    argc,
      char **argv)
{
  int rv = 1;
  
  g_type_init ();
  
  if (argc >= 2) {
    GError           *err = NULL;
    GFile            *file;
    CtplInputStream  *stream;
    
    file = g_file_new_for_commandline_arg (argv[1]);
    stream = ctpl_input_stream_new_for_gfile (file, &err);
    g_object_unref (file);
    if (! stream) {
      fprintf (stderr, "Failed to open input: %s\n", err->message);
    } else {
      CtplToken  *root;
      
      root = ctpl_lexer_lex (stream, &err);
      if (! root) {
        fprintf (stderr, "Wrong data: %s\n", err->message);
      } else {
        CtplOutputStream *output;
        GOutputStream    *gostream;
        
        gostream = g_memory_output_stream_new (NULL, 0, realloc, free);
        output = ctpl_output_stream_new (gostream);
        if (! ctpl_parser_parse (root, NULL, output, &err)) {
          fprintf (stderr, "Parser failed: %s\n", err->message);
        } else {
          dump_g_memory_output_stream (G_MEMORY_OUTPUT_STREAM (gostream), stdout);
          rv = 0;
        }
        g_object_unref (gostream);
        ctpl_output_stream_unref (output);
      }
      ctpl_lexer_free_tree (root);
    }
    ctpl_input_stream_unref (stream);
    if (err) {
      g_error_free (err);
    }
  }
  
  return rv;
}
